#include <stdlib.h>

#define NIL_POINTER			( (void *)0L)
#define MOVE_TO_FRONT		-1L
#define REMOVE_ALL_EVENTS	0
#define NIL_STRING			"\p"
#define NIL_TITLE			NIL_STRING
#define NIL_MOUSE_REGION	NIL_POINTER
#define VISIBLE				(1==1)
#define NO_GO_AWAY			(1==0)
#define NIL_REF_CON			NIL_POINTER

#define SLEEP_TIME			2L
#define UPDATE_TIME			4L
#define FRICTION			7
#define CLOSE				50

#define WNE_TRAP_NUM		0x60
#define UNIMPL_TRAP_NUM		0x9f

#define MEDIUM_SCREEN_RIGHT	648

#define SMALL_SCREEN		502
#define MEDIUM_SCREEN		501
#define LARGE_SCREEN		500
#define APPLE_BASIC			510
#define FILE_BASIC			520
#define EDIT_BASIC			530
#define VIEW_BASIC			540
#define SPECIAL_BASIC		550
#define FINDER_BASIC		560
#define DISK_BASIC			570
#define TRASH_BASIC			580

#define EXIT_ALERT			300

#define APPLE_PICT			0
#define FILE_PICT			1
#define EDIT_PICT			2
#define VIEW_PICT			3
#define SPECIAL_PICT		4
#define FINDER_PICT			5
#define DISK_PICT			6
#define TRASH_PICT			7

#define NUMBER_VARIANTS		2
#define NUMBER_PICTS		8

typedef struct tAPict
{
	PicHandle	thePicture[NUMBER_VARIANTS];
	int			currentVariant;
	Rect		currentLocation;
	int			hDelta[NUMBER_VARIANTS], vDelta[NUMBER_VARIANTS];
	int			lastXMove, lastYMove;
	int			friction;
	int			closeDistance;
} tAPict;

typedef enum {ISTRUE, ISFALSE} tBoolean;

void		main(void);
void		ToolBoxInit(void);
void		WindowInit(void);
void		PictInit(void);
void		MainLoop(void);
void		InitOnePict(int, int, int, int, int, int);
void		HandleEvent(void);
void		DrawTheIcons(void);
void		RedrawEverything(void);
void		DrawOneIcon(int, Point);
tBoolean	MouseIsClose(int, Point);
void		ComputeVelocity(int, Point);

tAPict		gTheBackground;
tAPict		gThePict[NUMBER_PICTS];
int			gMBarHeightSave;
WindowPtr	gSpooferWindow;
tBoolean	gDone=ISFALSE;
tBoolean	gMultiFinder;
EventRecord	gTheEvent;
tBoolean	gStartAvoid=ISFALSE;

void main()
{
	ToolBoxInit();
	WindowInit();
	PictInit();
	MainLoop();
}

void ToolBoxInit()
{
	InitGraf(&thePort);
	InitFonts();
	FlushEvents(everyEvent, REMOVE_ALL_EVENTS);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NIL_POINTER);
	InitCursor();
}

void WindowInit()
{
	Rect		tempRect;
	RgnHandle	tempRgn;
	
	gMBarHeightSave = MBarHeight;
	MBarHeight = 0;
	gSpooferWindow = NewWindow(NIL_POINTER, &(screenBits.bounds), NIL_TITLE,
						VISIBLE, plainDBox, MOVE_TO_FRONT, NO_GO_AWAY,
						NIL_REF_CON);
	if (gSpooferWindow == NIL_POINTER)
	{
		MBarHeight = gMBarHeightSave;
		ParamText("\pCouldn't create Spoofer window",NIL_STRING, NIL_STRING,
			NIL_STRING);
		StopAlert(EXIT_ALERT, NIL_POINTER);
		ClipRect(&(screenBits.bounds));
		exit(0);
	}
	SetRect(&tempRect, screenBits.bounds.left, screenBits.bounds.top,
			screenBits.bounds.right, screenBits.bounds.top+gMBarHeightSave);
	tempRgn = NewRgn();
	RectRgn(tempRgn, &tempRect);
	UnionRgn(gSpooferWindow->visRgn, tempRgn, gSpooferWindow->visRgn);
	DisposeRgn(tempRgn);
	SetPort(gSpooferWindow);
	ClipRect(&(screenBits.bounds));
	FillRect(&(gSpooferWindow->portRect),black);
	PenMode(patXor);
}

void PictInit()
{
	if (screenBits.bounds.right < MEDIUM_SCREEN_RIGHT - 72)
		gTheBackground.thePicture[0] = GetPicture(SMALL_SCREEN);
	else if (screenBits.bounds.right > MEDIUM_SCREEN_RIGHT + 72)
		gTheBackground.thePicture[0] = GetPicture(LARGE_SCREEN);
	else gTheBackground.thePicture[0] = GetPicture(MEDIUM_SCREEN);
	if (gTheBackground.thePicture[0] == NIL_POINTER)
	{
		MBarHeight = gMBarHeightSave;
		ParamText("\pCouldn't get the background picture",NIL_STRING, NIL_STRING,
			NIL_STRING);
		StopAlert(EXIT_ALERT, NIL_POINTER);
		ClipRect(&(screenBits.bounds));
		exit(0);
	}
	gTheBackground.currentLocation = screenBits.bounds;
	DrawPicture(gTheBackground.thePicture[0], &(gTheBackground.currentLocation));
	
	InitOnePict(APPLE_PICT,   APPLE_BASIC,   2, 19,  FRICTION, CLOSE);
	InitOnePict(FILE_PICT,    FILE_BASIC,    2, 43,  FRICTION, CLOSE);
	InitOnePict(EDIT_PICT,    EDIT_BASIC,    2, 79,  FRICTION, CLOSE);
	InitOnePict(VIEW_PICT,    VIEW_BASIC,    3, 117, FRICTION, CLOSE);
	InitOnePict(SPECIAL_PICT, SPECIAL_BASIC, 2, 162, FRICTION, CLOSE);
	InitOnePict(FINDER_PICT,  FINDER_BASIC,  -1,
		gSpooferWindow->portRect.right - 30,  FRICTION, CLOSE);
	InitOnePict(DISK_PICT,    DISK_BASIC,    46,
		gSpooferWindow->portRect.right - 56,  FRICTION, CLOSE);
	InitOnePict(TRASH_PICT,   TRASH_BASIC,
		gSpooferWindow->portRect.bottom - 60,
		gSpooferWindow->portRect.right  - 56, FRICTION, CLOSE);
}
	
void InitOnePict(int number, int resource, int top, int left, int fric, int close)
{
	int i;
	
	gThePict[number].lastXMove = gThePict[number].lastYMove = 0;
	gThePict[number].friction = fric;
	gThePict[number].closeDistance = close;
	
	for (i=0; i<NUMBER_VARIANTS; ++i)
	{
		gThePict[number].thePicture[i] = GetPicture(resource + i);
		if (gThePict[number].thePicture[i] == NIL_POINTER)
		{
			MBarHeight = gMBarHeightSave;
			ParamText("\pCouldn't initialize a small picture",NIL_STRING, NIL_STRING,
				NIL_STRING);
			StopAlert(EXIT_ALERT, NIL_POINTER);
			ClipRect(&(screenBits.bounds));
			exit(0);
		}
		gThePict[number].hDelta[i] =
			(*(gThePict[number].thePicture[i]))->picFrame.right -
			(*(gThePict[number].thePicture[i]))->picFrame.left;
		gThePict[number].vDelta[i] =
			(*(gThePict[number].thePicture[i]))->picFrame.bottom -
			(*(gThePict[number].thePicture[i]))->picFrame.top;
	}
	gThePict[number].currentVariant = 0;
	gThePict[number].currentLocation.top = top;
	gThePict[number].currentLocation.left = left;
	gThePict[number].currentLocation.bottom =
		gThePict[number].currentLocation.top + gThePict[number].vDelta[0];
	gThePict[number].currentLocation.right =
		gThePict[number].currentLocation.left + gThePict[number].hDelta[0];
	DrawPicture(gThePict[number].thePicture[0], &(gThePict[number].currentLocation));
}

void MainLoop()
{
	gDone = ISFALSE;
	if (NGetTrapAddress(WNE_TRAP_NUM, ToolTrap) !=
			NGetTrapAddress(UNIMPL_TRAP_NUM, ToolTrap))
		gMultiFinder = ISTRUE;
	else gMultiFinder = ISFALSE;
	while (gDone == ISFALSE) HandleEvent();
}

void HandleEvent()
{
	Rect		tempRect;
	RgnHandle	tempRgn, tempRgn2;
	
	if (gMultiFinder) WaitNextEvent(everyEvent, &gTheEvent, SLEEP_TIME,
						NIL_MOUSE_REGION);
	else
	{
		SystemTask();
		GetNextEvent(everyEvent, &gTheEvent);
	}
	if (gTheEvent.what == mouseDown) gStartAvoid = ISTRUE;
	switch(gTheEvent.what)
	{
		case keyDown:
		case keyUp:
		case autoKey:
			if ( (((gTheEvent.message & charCodeMask) == 'm') ||
				  ((gTheEvent.message & charCodeMask) == 'M')) &&
				 ((gTheEvent.modifiers & cmdKey) != 0))
			{
				tempRgn = NewRgn(); tempRgn2 = NewRgn();
				SetRectRgn(tempRgn2, screenBits.bounds.left, screenBits.bounds.top,
					screenBits.bounds.right, screenBits.bounds.top+gMBarHeightSave);
				DiffRgn(gSpooferWindow->visRgn, tempRgn2, tempRgn);
				CopyRgn(tempRgn, gSpooferWindow->visRgn);
				DisposeRgn(tempRgn); DisposeRgn(tempRgn2);
				MBarHeight = gMBarHeightSave;
				break;
			}
			if ((gTheEvent.modifiers & cmdKey) != 0)
			{
				MBarHeight = gMBarHeightSave;
				/* ParamText("\pNormal exit",NIL_STRING, NIL_STRING,
					NIL_STRING);
				StopAlert(EXIT_ALERT, NIL_POINTER); */
				ClipRect(&(screenBits.bounds));
				exit(0);
			}
		case activateEvt:
		case updateEvt:
			RedrawEverything();
		case nullEvent:
		case mouseDown:
		case mouseUp:
		case diskEvt:
		case networkEvt:
		case driverEvt:
		case app1Evt:
		case app2Evt:
		case app3Evt:
		case app4Evt:
		default:
			DrawTheIcons();
	}
}

void RedrawEverything()
{
	int i;
	Rect tempRect;
	RgnHandle tempRgn;
	
	MBarHeight = 0;
	SetRect(&tempRect, screenBits.bounds.left, screenBits.bounds.top,
		screenBits.bounds.right, screenBits.bounds.top+gMBarHeightSave);
	tempRgn = NewRgn();
	RectRgn(tempRgn, &tempRect);
	UnionRgn(gSpooferWindow->visRgn, tempRgn, gSpooferWindow->visRgn);
	DisposeRgn(tempRgn);
	SetPort(gSpooferWindow);
	ClipRect(&(screenBits.bounds));
	SelectWindow(gSpooferWindow);
	PenMode(patCopy);
	BeginUpdate(gTheEvent.message);
	EndUpdate(gTheEvent.message);
	DrawPicture(gTheBackground.thePicture[0], &(gTheBackground.currentLocation));
	for (i=0; i<NUMBER_PICTS; ++i)
		DrawPicture(gThePict[i].thePicture[gThePict[i].currentVariant],
			&(gThePict[i].currentLocation));
	PenMode(patXor);
}


long int gLastEventTime = 0L;

void DrawTheIcons()
{
	int i;
	Point theLoc;
	
	if (gStartAvoid == ISFALSE) return;
	if (gLastEventTime + UPDATE_TIME > TickCount()) return;
	gLastEventTime = TickCount();

	GetMouse(&theLoc);
	for (i=0; i<NUMBER_PICTS; ++i)
	{
		if ( (gThePict[i].currentVariant != 0) ||
			 (gThePict[i].lastXMove != 0) ||
			 (gThePict[i].lastYMove != 0) ||
			 (MouseIsClose(i, theLoc) == ISTRUE) )
			DrawOneIcon(i,theLoc);
	}
}

tBoolean MouseIsClose(int n, Point p)
{
	int vLoc=	(gThePict[n].currentLocation.top +
				 gThePict[n].currentLocation.bottom)/2;
	int hLoc=	(gThePict[n].currentLocation.left +
				 gThePict[n].currentLocation.right)/2;
	
	if (abs(p.v - vLoc) + abs(p.h - hLoc) < gThePict[n].closeDistance)
		return(ISTRUE);
	else return(ISFALSE);
}

void DrawOneIcon(int n, Point p)
{
	ClipRect(&(gThePict[n].currentLocation));
	DrawPicture(gTheBackground.thePicture[0], &(gTheBackground.currentLocation));
	ComputeVelocity(n,p);
	++gThePict[n].currentVariant;
	if (gThePict[n].currentVariant >= NUMBER_VARIANTS)
		gThePict[n].currentVariant = 0;
	
	gThePict[n].currentLocation.top  += gThePict[n].lastYMove;
	if (gThePict[n].currentLocation.top < 0)
		gThePict[n].currentLocation.top = 0;
	else if ( (gThePict[n].currentLocation.top+gThePict[n].currentLocation.bottom)/2 >
						gTheBackground.currentLocation.bottom)
		gThePict[n].currentLocation.top =
			gTheBackground.currentLocation.bottom - 
			gThePict[n].vDelta[gThePict[n].currentVariant];
	
	gThePict[n].currentLocation.left += gThePict[n].lastXMove;
	if (gThePict[n].currentLocation.left < 0)
		gThePict[n].currentLocation.left = 0;
	else if (gThePict[n].currentLocation.left > gTheBackground.currentLocation.right)
		gThePict[n].currentLocation.left =
			gTheBackground.currentLocation.right -
			gThePict[n].hDelta[gThePict[n].currentVariant];
	
	gThePict[n].currentLocation.bottom =
		gThePict[n].currentLocation.top +
		gThePict[n].vDelta [gThePict[n].currentVariant];
	gThePict[n].currentLocation.right =
		gThePict[n].currentLocation.left +
		gThePict[n].hDelta [gThePict[n].currentVariant];
	
	ClipRect(&(gThePict[n].currentLocation));
	DrawPicture(gThePict[n].thePicture [gThePict[n].currentVariant],
		&(gThePict[n].currentLocation));
}

void ComputeVelocity(int n, Point p)
{
	int xAccum,yAccum,i,j;
	int hLoc=(gThePict[n].currentLocation.left+gThePict[n].currentLocation.right)/2;
	int vLoc=(gThePict[n].currentLocation.top+gThePict[n].currentLocation.bottom)/2;
	
	/* if (MouseIsClose(n,p) == ISFALSE)
	{
		gThePict[n].lastXMove = 0;
		gThePict[n].lastYMove = 0;
		return;
	} */
	
	/* Back out friction effects */
	if (gThePict[n].lastXMove > 0)
		if (gThePict[n].lastXMove > gThePict[n].friction)
			xAccum = gThePict[n].lastXMove - gThePict[n].friction;
		else xAccum = 0;
	else
		if (gThePict[n].lastXMove < -gThePict[n].friction)
			xAccum = gThePict[n].lastXMove + gThePict[n].friction;
		else xAccum = 0;
	if (gThePict[n].lastYMove > 0)
		if (gThePict[n].lastYMove > gThePict[n].friction)
			yAccum = gThePict[n].lastYMove - gThePict[n].friction;
		else yAccum = 0;
	else
		if (gThePict[n].lastYMove < -gThePict[n].friction)
			yAccum = gThePict[n].lastYMove + gThePict[n].friction;
		else yAccum = 0;
	if ( (xAccum == 0) && (yAccum == 0) && (MouseIsClose(n,p)==ISFALSE))
	{
		gThePict[n].lastXMove = 0;
		gThePict[n].lastYMove = 0;
		return;
	}
	
	if (hLoc <= 1) hLoc = 1;
	if (vLoc <= 1) vLoc = 1;
	
	/* the sides have a repulsive force */
	i = hLoc + vLoc;
	if (i > 10) i = 10;
	i = 100/(i*i);
	xAccum += (i * hLoc)/(hLoc + vLoc);
	yAccum += (i * vLoc)/(hLoc + vLoc);
	
	/* The mouse also repels */
	j = abs(p.h - hLoc) + abs(p.v - vLoc);
	if (j <= 1)
	{
		xAccum += screenBits.bounds.right/2  - hLoc;
		yAccum += screenBits.bounds.bottom/2 - vLoc;
	}
	else
	{
		i = 200/j;
		xAccum -= (i*(p.h-hLoc))/j;
		yAccum -= (i*(p.v-vLoc))/j;
	}
	
	/* The center attracts */
	j = abs(hLoc-screenBits.bounds.right/2)+abs(vLoc-screenBits.bounds.bottom/2);
	i = j/70;
	xAccum -= (i*(hLoc-screenBits.bounds.right/2))/j;
	yAccum -= (i*(vLoc-screenBits.bounds.bottom/2))/j;

	if (xAccum > 0)
	{
		if (xAccum > 50) xAccum = 50;
	}
	else if (xAccum < -50) xAccum = -50;
	if (yAccum > 0)
	{
		if (yAccum > 50) yAccum = 50;
	}
	else if (yAccum < -50) yAccum = -50;
	
	gThePict[n].lastXMove = xAccum;
	gThePict[n].lastYMove = yAccum;
}

