// Copyright  1992-1993 Emergent Behavior. All rights reserved.

//=================================================================
#ifndef _TLANDERVIEW_
	#include "TLanderView.h"
#endif

#ifndef _TLANDERDOCUMENT_
	#include "TLanderDocument.h"
#endif

//=================================================================
TLanderView::TLanderView(Point size, TDocument* theDoc, Scalar initialAltitude)
	: 	TBufferedView(size, theDoc), 
		fInitialAltitude(initialAltitude),
		fThrust( TThrustForce(0, 0) ),
		fBackgroundPicture(NIL), 
		fShipPicture(NIL),
		fAltitudeText(NIL), 
		fSpeedText(NIL),
		fFuelText(NIL), 
		fThrustText(NIL),
		fTimeText(NIL)
{
}

//-----------------------------------------------------------------
TLanderView::~TLanderView()
{
}

//-----------------------------------------------------------------
TSystem*
TLanderView::GetSystem()
{
	return ((TLanderDocument*)fDocument)->GetSystem();
}

//-----------------------------------------------------------------
TSESpaceship*
TLanderView::GetShip()
{
	TListIterator iter = this->GetSystem()->GetBodyIterator();
	return (TSESpaceship*)iter.First();
}

//-----------------------------------------------------------------
void
TLanderView::DoMakeItems()
{
	const Point kOrigin = {0, 0};
	
	const short kBackgroundResID = 1001;
	const short kShipResID = 1002;
	
	const Rect kAltitudeFrame 	= {223,  80, 239, 130};	// y, x, y, x
	const Rect kSpeedFrame 		= {250,  80, 266, 130};	// y, x, y, x
	
	const Rect kFuelFrame 		= {214, 315, 231, 380};	// y, x, y, x
	const Rect kThrustFrame 	= {241, 315, 256, 380};	// y, x, y, x
	const Rect kTimeFrame 		= {267, 315, 283, 380};	// y, x, y, x

	fBackgroundPicture = new TPicture(kBackgroundResID, kOrigin, this);
	fItemList.AdoptItem(fBackgroundPicture);

	fShipPicture = new TPicture(kShipResID, kOrigin, this);
	fItemList.AdoptItem(fShipPicture);

	fAltitudeText = new TText(kAltitudeFrame, this);
	fItemList.AdoptItem(fAltitudeText);

	fSpeedText = new TText(kSpeedFrame, this);
	fItemList.AdoptItem(fSpeedText);

	fFuelText = new TText(kFuelFrame, this);
	fItemList.AdoptItem(fFuelText);

	fThrustText = new TText(kThrustFrame, this);
	fItemList.AdoptItem(fThrustText);

	fTimeText = new TText(kTimeFrame, this);
	fItemList.AdoptItem(fTimeText);
}

//-----------------------------------------------------------------
void
TLanderView::UpdateValues()
{
	const float kScaleFactor  = (float)200/fInitialAltitude;		// pixels/meters
	TSESpaceship* ship = this->GetShip();
	TRealPoint location = ship->GetLocation();
	Scalar xLocation = location.GetHorizontal();
	Scalar yLocation = location.GetVertical();
	fShipPicture->SetLocation( 	(short)(xLocation*kScaleFactor), 
								(short)(yLocation*kScaleFactor - kShipHeight) );
	long altitude	= (long)( (fInitialAltitude - yLocation));
	fAltitudeText->SetText( altitude );

	TVector velocity = ship->GetVelocity();
	fSpeedText->SetText( (long) (velocity.GetLength()) );

	Mass fuel = ship->GetRemainingFuel();
	fFuelText->SetText( long(fuel) );

	long thrustMag = (long) fThrust.GetVertical();
	fThrustText->SetText(thrustMag);
	
	long elapsedTicks = TickCount() - ((TLanderDocument*)fDocument)->GetInitialTicks();
	fTimeText->SetText( (long) (elapsedTicks/60.0) );
	
	if ( altitude < 1 ) {
		((TLanderDocument*)fDocument)->SetMissionInProgress(false);
		((TLanderDocument*)fDocument)->SetFinalSpeed(velocity.GetLength());
	}
}

//-----------------------------------------------------------------
void
TLanderView::DoKeyCommand(char key)
{
	float kTimeStepSize	= 0.25;					// sec
	short value = key - 48;
	if (( value > -1) && (value <10) ) {
		Scalar newThrustValue = value * 5.0e4;
		fThrust.SetVertical( - newThrustValue);
		((TLanderDocument*)fDocument)->SetThrust(fThrust);
		}
}

//-----------------------------------------------------------------
void
TLanderView::Draw()
{
	this->UpdateValues();
}
